/*
Copyright (c) 2007, Mike Kinney
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the 
following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the 
following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the 
following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/
//
//  MKAsyncRequest.h
//  MKAbeFook
//
//  Created by Mike on 3/8/07.
//  Copyright 2007 Mike Kinney. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "MKFacebook.h"
#import "MKFacebookMethods.h"

@interface MKAsyncRequest : NSObject {
	MKFacebook *facebookConnection;
	id _delegate;
	SEL _selector;
	NSMutableData *responseData;
	BOOL doingDownload;
}

-(MKAsyncRequest *)initWithFacebookConnection:(MKFacebook *)aFacebookConnection 
									 delegate:(id)aDelegate 
									 selector:(SEL)aSelector;


#pragma mark  Returning Methods
#pragma mark FQL
+(void)facebookFQLQuery:(NSString *)aQuery 
	 facebookConnection:(MKFacebook *)aFacebookConnection 
			   delegate:(id)aDelegate 
			   selector:(SEL)aSelector;

#pragma mark -
#pragma mark Events
+(void)facebookEventsGet:(int)aUid 
					eids:(NSArray *)eids 
			   startTime:(int)startTime 
				 endTime:(int)endTime 
			  rsvpStatus:(NSString *)rsvpStatus 
				facebookConnection:(MKFacebook *)aFacebookConnection
				delegate:(id)aDelegate 
				selector:(SEL)aSelector;

+(void)facebookEventsGetMembers:(NSString *)eid 
			 facebookConnection:(MKFacebook *)aFacebookConnection	
					   delegate:(id)aDelegate 
					   selector:(SEL)aSelector;
#pragma mark -

#pragma mark Friends
+(void)facebookFriendsGet:(MKFacebook *)aFacebookConnection
				 delegate:(id)aDelegate 
				 selector:(SEL)aSelector;

+(void)facebookFriendsGetAppUsers:(MKFacebook *)aFacebookConnection 
						 delegate:(id)aDelegate 
						 selector:(SEL)aSelector;
+(void)facebookFriendsAreFriends:(NSArray *)id1 
							 id2:(NSArray *)id2
			  facebookConnection:(MKFacebook *)aFacebookConnection 
						delegate:(id)aDelegate 
						selector:(SEL)aSelector;
#pragma mark -

#pragma mark Groups
+(void)facebookGroupsGet:(int)aUid 
					gids:(NSArray *)gids 
	  facebookConnection:(MKFacebook *)aFacebookConnection 		
				delegate:(id)aDelegate 
				selector:(SEL)aSelector;
+(void)facebookGroupsGetMembers:(NSString *)gid 
			 facebookConnection:(MKFacebook *)aFacebookConnection 
					   delegate:(id)aDelegate 
					   selector:(SEL)aSelector;
#pragma mark -

#pragma mark Notifications
+(void)facebookNotificationsGet:(MKFacebook *)aFacebookConnection 
					   delegate:(id)aDelegate 
					   selector:(SEL)aSelector;
#pragma mark -

#pragma mark Photos
+(void)facebookPhotosGetCommentCountXML:(MKFacebook *)aFacebookConnection 
							   delegate:(id)aDelegate 
							   selector:(SEL)aSelector;

+(void)facebookPhotosAddTag:(NSString *)aPid 
					 tagUid:(int)aTagUid 
					tagText:(NSString *)aTagText 
						  x:(float)aX 
						  y:(float)aY 
		 facebookConnection:(MKFacebook *)aFacebookConnection 
				   delegate:(id)aDelegate 
				   selector:(SEL)aSelector;

+(void)facebookPhotosCreateAlbum:(NSString *)aName 
						location:(NSString *)aLocation 
					 description:(NSString *)aDescription
			  facebookConnection:(MKFacebook *)aFacebookConnection 
						delegate:(id)aDelegate 
						selector:(SEL)aSelector;

+(void)facebookPhotosGet:(int)subjId 
					 aid:(NSString *)aid 
					pids:(NSArray *)pids 
	  facebookConnection:(MKFacebook *)aFacebookConnection 			
				delegate:(id)aDelegate 
				selector:(SEL)aSelector;

+(void)facebookPhotosGetAlbums:(int)aUid 
						  pids:(NSArray *)pids 
			facebookConnection:(MKFacebook *)aFacebookConnection 
					  delegate:(id)aDelegate 
					  selector:(SEL)aSelector;
+(void)facebookPhotosGetTags:(NSArray *)pids 
		  facebookConnection:(MKFacebook *)aFacebookConnection 	
					delegate:(id)aDelegate 
					selector:(SEL)aSelector;
#pragma mark -

#pragma mark Update
+(void)facebookUpdateDecodeIDs:(NSArray *)ids 
			facebookConnection:(MKFacebook *)aFacebookConnection 				  
					  delegate:(id)aDelegate 
					  selector:(SEL)aSelector;
#pragma mark -

#pragma mark Users
+(void)facebookUsersGetInfo:(NSArray *)uids 
					 fields:(NSArray *)fields 
		 facebookConnection:(MKFacebook *)aFacebookConnection 				   
				   delegate:(id)aDelegate 
				   selector:(SEL)aSelector;

+(void)facebookUsersGetLoggedInUser:(MKFacebook *)aFacebookConnection 
						   delegate:(id)aDelegate 
						   selector:(SEL)aSelector;
#pragma mark -




@end
