//
//  Downloader.m
//  originally developed for use in FriendSaver
//
//  Created by Alex Roche on 3/26/07. Modified by Paul Borokhov for use in FMenu, Apr-May 2007.
//  Copyright 2007 Alex Roche. All rights reserved.
//  Portions copyright 2007-2009 Paul Borokhov

#import "Downloader.h"
#ifndef MKFacebook
#import <MKAbeFook/MKAbeFook.h>
#endif

#pragma mark C code

void downloaderNetworkStatusDidChange(SCNetworkReachabilityRef name, SCNetworkConnectionFlags flags, void * infoDictionary) {
	// code from http://git.symonds.id.au/gitweb.cgi?p=marcopolo.git;a=blob;f=src/IPEvidenceSource.m;hb=HEAD used to figure out how to do this correctly. Thanks to David Symonds from cocoa-dev@lists.apple for the help.
	if (name != NULL && infoDictionary != NULL) {
		Downloader *info = (Downloader*) infoDictionary;
		if (flags != kSCNetworkFlagsReachable) {
			[info willGoToSleepSoon:nil];
		}
	}
}

@implementation Downloader

-(id)initWithRequest:(NSURLRequest *)theRequest {
	if ((self = [super init])) {
		notificationRegistered = NO;
		[[NSURLCache sharedURLCache] setMemoryCapacity:0];
		[[NSURLCache sharedURLCache] setDiskCapacity:0];
		facebookName = SCNetworkReachabilityCreateWithName(kCFAllocatorDefault, [@"www.facebook.com" cStringUsingEncoding:NSISOLatin1StringEncoding]);
		[self setInitialAvailability];
		receivedData = nil;

		if (!isReachable) {
			return [self dieCleanly];
		}
		
        NSURLResponse *urlResponse = nil;
		// begin the connection
#if __MAC_OS_X_VERSION_MAX_ALLOWED < __MAC_OS_X_VERSION_10_5
		connection = [[NSURLConnection connectionWithRequest:theRequest delegate:self] retain];
#else
        receivedData = [NSURLConnection sendSynchronousRequest:theRequest returningResponse:&urlResponse error:NULL];
#endif
		
		if (urlResponse) {
			doingDownload = YES;
			hasError = NO;
#if __MAC_OS_X_VERSION_MAX_ALLOWED < __MAC_OS_X_VERSION_10_5
			// TODO: somehow make sure this actually succeeds...
			[[[NSWorkspace sharedWorkspace] notificationCenter] addObserver:self selector:@selector(willGoToSleepSoon:) name:NSWorkspaceWillSleepNotification object:nil];
			
			SCNetworkReachabilityContext info;
			info.version = 0;
			info.info = self;
			info.retain = NULL;
			info.release = NULL;
			info.copyDescription = NULL;
			
			current = (CFRunLoopRef) CFRetain(CFRunLoopGetCurrent());
			
			if (SCNetworkReachabilitySetCallback(facebookName, downloaderNetworkStatusDidChange, &info) &&
				SCNetworkReachabilityScheduleWithRunLoop(facebookName, current, kCFRunLoopDefaultMode)) {
				// damn, when did we start writing C code?
				notificationRegistered = YES;
			}
			
			receivedData = [[NSMutableData alloc] init];
			// NSLog(@"Connection created");
			
			double resolution = 1.0;
			while (doingDownload) {
				NSDate *next = [NSDate dateWithTimeIntervalSinceNow:resolution]; 
				[[NSRunLoop currentRunLoop] runUntilDate:next];
			}
#else
            [self connectionDidFinishLoading:connection];
#endif
			return self;
		} else {
			hasError = YES;
			doingDownload = NO;
			if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
				NSLog(@"could not create connection...");
			}
			return [self dieCleanly];
		}
	} else {
		return nil;
	}
}

-(id)dieCleanly {
	if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"] && [[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logDownloaderDeath"]) {
		NSLog(@"Downloader object committing self-destruction...");
	}
	[self release];
	// this feels a bit sketchy.
	return nil;
}

-(void)dealloc {
	// NSLog(@"deallocing a Downloader...");
	if (notificationRegistered) {
		//[[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver:self];
		[[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver:self	name:NSWorkspaceWillSleepNotification object:nil];
		SCNetworkReachabilityUnscheduleFromRunLoop(facebookName, current, kCFRunLoopDefaultMode);
		CFRelease(current);
	}
#if __MAC_OS_X_VERSION_MAX_ALLOWED < __MAC_OS_X_VERSION_10_5
	[receivedData release];
	[connection release];
#endif
	CFRelease(facebookName);
	[super dealloc];
	// NSLog(@"...done");
}

- (void)setInitialAvailability {
	SCNetworkConnectionFlags reachability;
	SCNetworkReachabilityGetFlags(facebookName, &reachability);
	if (reachability != kSCNetworkFlagsReachable) {
		isReachable = NO;
	} else {
		isReachable = YES;
	}
}

-(void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	[receivedData setLength:0];
}

-(void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	// NSLog(@"received data");
	[receivedData appendData:data];
}

-(void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	// NSLog(@"connection error..");
	hasError = YES;
	doingDownload = NO;
	if ([[[NSUserDefaultsController sharedUserDefaultsController] defaults] boolForKey:@"logErrors"]) {
		NSLog(@"Connection failed! Error - %@, URL %@", [error localizedDescription], [[error userInfo] objectForKey:NSErrorFailingURLStringKey]);
	}
}

-(NSMutableData *)data {
	if (receivedData) {
		return [[receivedData copy] autorelease];
	} else {
		return nil;
	}
}

-(BOOL)hasError {
	return hasError;
}

-(void)connectionDidFinishLoading:(NSURLConnection *)connection {
	//NSLog(@"received data");
	doingDownload = NO;
	if (![MKFacebook validXMLResponse:[[[NSXMLDocument alloc] initWithData:receivedData options:0 error:nil] autorelease]] || 
		[receivedData length] == 0) {
		hasError = YES;
	}
}

-(NSCachedURLResponse *)connection:(NSURLConnection *)connection willCacheResponse:(NSCachedURLResponse *)cachedResponse {
	return nil;
}

-(void)willGoToSleepSoon:(id)anObject {
	if (doingDownload) {
		// NSLog(@"cancelling download on wake");
		[connection cancel];
		hasError = YES;
		doingDownload = NO;
	}
}

@end